from django import template
from armstrong.core.arm_content.images.sorl import get_preset_thumbnail

register = template.Library()


@register.filter
def thumbnail(value, arg):
    return get_preset_thumbnail(value, arg).url



from .. import settings


def inject_defaults(func):
    def inner(self, embed, width=None, height=None, **kwargs):
        if not width:
            width = settings.ARMSTRONG_EMBED_VIDEO_WIDTH
        if not height:
            height = settings.ARMSTRONG_EMBED_VIDEO_HEIGHT
        return func(self, embed, width, height, **kwargs)
    return inner



from ._utils import *
import datetime
from django.db import models
from django import template
from taggit.managers import TaggableManager

from .arm_content_support.models import SorlImage

from armstrong.core.arm_content.templatetags import content_helpers


class ThumbnailTestCase(ArmContentTestCase):
    def test_thumbnail_filter(self):
        thumb_url = "http://example.com/thumbnail_url.jpg"
        obj = { 'image': "image"}
        t = template.Template("{% load content_helpers %}{{ obj.image|thumbnail:'thumb_size' }}")

        thumbnail_result = fudge.Fake()
        thumbnail_result.has_attr(url=thumb_url)

        get_preset_thumbnail = fudge.Fake()
        get_preset_thumbnail.expects_call().\
                with_args("image", u'thumb_size').\
                returns(thumbnail_result)
        with fudge.patcher.patched_context(content_helpers, 
                    'get_preset_thumbnail',
                    get_preset_thumbnail):
            result = t.render(template.Context({'obj': obj}))
        self.assertEqual(result, thumb_url)
        fudge.verify()



from .._utils import *
from django.core.urlresolvers import reverse

from .fieldsets import *


class AuthorsAdminTest(ArmContentTestCase):
    fixtures = ['admin.json', ]

    def test_authors_fields_are_listed_as_such(self):
        self.client.login(username="admin", password="admin")
        url = reverse("admin:arm_content_support_simpleauthoredmodel_add")
        response = self.client.get(url)
        label_pattern = r'<label [^>]*for="id_authors"[^>]*>Authors:</label>'
        self.assertRegexpMatches(str(response), label_pattern)



from django.conf import settings


ARMSTRONG_EMBED_VIDEO_HEIGHT = getattr(settings,
        'ARMSTRONG_EMBED_VIDEO_HEIGHT', 390)
ARMSTRONG_EMBED_VIDEO_WIDTH = getattr(settings,
        'ARMSTRONG_EMBED_VIDEO_WIDTH', 640)



import fudge
from fudge.inspector import arg
import random
from ..._utils import *

from .youtube import *
from .vimeo import *

from ....video import backends
from ....video.backends import helpers
from armstrong.utils.backends.base import MultipleBackendProxy
from armstrong.utils.backends import base


class TestableBackend(object):
    pass


class SomeOtherTestableBackend(object):
    pass


class GetBackendTestCase(ArmContentTestCase):
    def backend_name(self, name):
        return "%s.%s" % (self.__module__, name)

    def test_uses_backend_from_settings(self):
        backend_name = self.backend_name('TestableBackend')
        settings = fudge.Fake()
        settings.has_attr(ARMSTRONG_EXTERNAL_VIDEO_BACKEND=backend_name)

        backends.backend.settings = settings
        backend = backends.get_backend()
        self.assertIsA(backend, TestableBackend)

    def test_uses_injected_settings_if_provided(self):
        backend_name = self.backend_name('SomeOtherTestableBackend')
        settings = fudge.Fake()
        settings.has_attr(ARMSTRONG_EXTERNAL_VIDEO_BACKEND=backend_name)
        backends.backend.settings = settings

        backend = backends.get_backend()
        self.assertIsA(backend, SomeOtherTestableBackend)

    def test_returns_MultipleBackend_if_configured_with_a_list(self):
        my_backends = [self.backend_name('TestableBackend'),
                self.backend_name('SomeOtherTestableBackend')]

        settings = fudge.Fake()
        settings.has_attr(ARMSTRONG_EXTERNAL_VIDEO_BACKEND=my_backends)
        backends.backend.settings = settings

        backend = backends.get_backend()
        self.assertIsA(backend, MultipleBackendProxy)


class InjectDefaultsDecoratorTestCase(ArmContentTestCase):
    def test_provides_default_width_and_height(self):
        random_width = random.randint(1000, 2000)
        random_height = random.randint(1000, 2000)
        embed = object()
        settings = fudge.Fake()
        settings.has_attr(ARMSTRONG_EMBED_VIDEO_WIDTH=random_width,
                ARMSTRONG_EMBED_VIDEO_HEIGHT=random_height)
        fake = fudge.Fake()
        fake.is_callable().with_args(self, embed, random_width, random_height)

        with fudge.patched_context(helpers, "settings", settings):
            foo = helpers.inject_defaults(fake)
            foo(self, embed)

    def test_kwargs_are_passed_through_to_backend(self):
        random_kwargs = dict(
                [("key_%d" % i, i) for i in range(random.randint(10, 20))])
        random_width = random.randint(1000, 2000)
        random_height = random.randint(1000, 2000)
        embed = object()
        settings = fudge.Fake()
        settings.has_attr(ARMSTRONG_EMBED_VIDEO_WIDTH=random_width,
                ARMSTRONG_EMBED_VIDEO_HEIGHT=random_height)
        fake = fudge.Fake()
        fake.is_callable().with_args(self, embed, random_width, random_height,
                **random_kwargs)

        with fudge.patched_context(helpers, "settings", settings):
            foo = helpers.inject_defaults(fake)
            foo(self, embed, **random_kwargs)



from __future__ import absolute_import
from django.db import models
from django.conf import settings

try:
    from sorl.thumbnail import ImageField
except ImportError:
    from django.db.models import ImageField

class BaseThumbnailMixin(object):
    '''Properties of an image-centric content item.'''
    visual_type = 'image'
    is_visual_embedded = False

    def render_visual(self, preset_label, presets=None, defaults=None, *args, **kwargs):
        raise NotImplementedError()

    def get_visual_thumbnail_url(self, preset_label, presets=None, defaults=None, *args, **kwargs):
        raise NotImplementedError()

UPLOAD_PATH = getattr(settings, 'ARMSTRONG_IMAGES_UPLOAD_PATH',
    'armstrong/images/')

class ImageMixin(models.Model):
    ''' Abstract mixin for images '''
    image = ImageField(upload_to=UPLOAD_PATH)

    class Meta:
        abstract = True

    def get_absolute_url(self):
        return self.image.url



import pkg_resources
pkg_resources.declare_namespace(__name__)

from armstrong.utils.backends import GenericBackend
from django.conf import settings as default_settings
from django.utils.importlib import import_module

from .youtube import YouTubeBackend
from .vimeo import VimeoBackend


backend = GenericBackend("ARMSTRONG_EXTERNAL_VIDEO_BACKEND")
get_backend = backend.get_backend



from copy import copy
from django.db import models
import fudge
from django.utils import unittest

try:
    import south
except ImportError:
    south = False

from .._utils import *
from ..arm_content_support.models import SimpleVideoModel

from ...fields.video import EmbeddedVideo
from ... import fields
from ...video import backends


class ExampleBackend(object):
    type = "Example"

    def prepare(self, embed):
        embed.url, embed.id = embed.raw_url.split(":")


class ExampleBackendTestCase(ArmContentTestCase):
    def test_type_of_Example(self):
        backend = ExampleBackend()
        self.assertEqual("Example", backend.type)

    def test_splits_on_colon(self):
        # This test assumes that EmbeddedVideo works
        random_url = "foobar.com/watch-%d" % random.randint(100, 200)
        random_id = "%d" % random.randint(100, 200)
        backend = ExampleBackend()
        video = EmbeddedVideo("%s:%s" % (random_url, random_id), backend)

        backend.prepare(video)
        self.assertEqual(video.url, random_url)
        self.assertEqual(video.id, random_id)


class EmbeddedVideoFieldTestCase(ArmContentTestCase):
    def test_sets_source_to_EmbeddedVideo(self):
        video = SimpleVideoModel()
        video.source = "http://www.youtube.com/watch?v=123"
        self.assertTrue(isinstance(video.source, EmbeddedVideo))

    def test_stores_and_retrieves_from_field_from_database(self):
        random_id = "abcdef%d" % random.randint(100, 200)
        video = SimpleVideoModel.objects.create(
                source="http://youtube.com/watch?v=%s" % random_id)
        id = video.pk
        self.assertEqual(random_id,
                SimpleVideoModel.objects.get(pk=id).source.id)

    def test_field_has_basic_label_by_default(self):
        field = fields.EmbeddedVideoField()
        self.assertEqual(field.formfield().label, u"Embedded Video URL")

    def test_field_can_have_custom_label_if_kwarg_provided(self):
        field = fields.EmbeddedVideoField()
        random_label = "Some random label: %d" % random.randint(100, 200)
        actual_field = field.formfield(label=random_label).label
        self.assertEqual(actual_field, random_label)

    @unittest.skipIf(south is False, "south not installed")
    def test_returns_expected_south_triple(self):
        field = fields.EmbeddedVideoField()

        expected = (
            "%s.%s" % (field.__class__.__module__, field.__class__.__name__),
            [],
            {'max_length': '255'}
        )
        self.assertEqual(field.south_field_triple(), expected)


class EmbeddedVideoTestCase(ArmContentTestCase):
    def setUp(self):
        fudge.clear_calls()
        fudge.clear_expectations()
        self.orig_backend_settings = copy(backends.backend.settings)

    def tearDown(self):
        backends.backend.settings = copy(self.orig_backend_settings)

    def test_id_is_None_by_default(self):
        v = EmbeddedVideo()
        self.assertEqual(None, v.id)

    def test_settings_a_source_to_a_YouTube_url_stores_the_source_id(self):
        random_id = "abcdef%d" % random.randint(100, 200)
        video = SimpleVideoModel()
        video.source = "http://www.youtube.com/watch?v=%s" % random_id
        self.assertEqual(random_id, video.source.id)

    def test_videos_on_YouTube_have_a_source_type_of_YouTube(self):
        random_id = "abcdef%d" % random.randint(100, 200)
        video = SimpleVideoModel()
        video.source = "http://www.youtube.com/watch?v=%s" % random_id
        self.assertEqual("YouTube", video.source.type)

    def test_can_take_an_EmbeddedVideo_object_as_a_type(self):
        v = EmbeddedVideo("http://www.youtube.com/watch?v=abc")
        video = SimpleVideoModel()
        video.source = v
        self.assertEqual("YouTube", video.source.type, msg="sanity check")
        self.assertEqual("abc", video.source.id, msg="sanity check")

    def test_uses_provided_backend(self):
        random_url = "foobar-%d" % random.randint(100, 200)
        random_id = "%d" % random.randint(100, 200)
        v = EmbeddedVideo("%s:%s" % (random_url, random_id),
                backend=ExampleBackend())
        self.assertEqual(random_id, v.id)
        self.assertEqual(random_url, v.url)
        self.assertEqual("Example", v.type)

    def test_uses_configured_backend_if_nothing_is_provided(self):
        settings = fudge.Fake()
        backend = "armstrong.core.arm_content.tests.fields.ExampleBackend"
        settings.has_attr(ARMSTRONG_EXTERNAL_VIDEO_BACKEND=backend)
        backends.backend.settings = settings

        random_url = "foobar-%d" % random.randint(100, 200)
        random_id = "%d" % random.randint(100, 200)
        v = EmbeddedVideo("%s:%s" % (random_url, random_id))
        self.assertEqual(random_id, v.id)
        self.assertEqual(random_url, v.url)

    def test_embed_dispatches_to_backend_and_returns_result(self):
        random_return = random.randint(1000, 2000)
        backend = fudge.Fake()
        backend.provides("prepare")
        video = EmbeddedVideo("foo/bar", backend=backend)
        backend.expects("embed").with_args(video).returns(random_return)
        fudge.clear_calls()

        result = video.embed()
        self.assertEqual(result, random_return)

        fudge.verify()

    def test_embed_dispatches_kwargs_to_backend(self):
        kwargs = dict(
                [("key-%d" % a, a) for a in range(random.randint(1, 10))])
        backend = fudge.Fake()
        backend.provides("prepare")
        video = EmbeddedVideo("foo/bar", backend=backend)
        backend.expects("embed").with_args(video, **kwargs)
        fudge.clear_calls()

        video.embed(**kwargs)
        fudge.verify()



from django.db import models

from ..video.backends import get_backend


class EmbeddedVideo(object):
    def __init__(self, url=None, backend=None):
        if not backend:
            backend = get_backend()
        self.backend = backend
        self.raw_url = url
        self.id = None
        if url:
            self.backend.prepare(self)

    @property
    def type(self):
        return self.backend.type

    def embed(self, **kwargs):
        return self.backend.embed(self, **kwargs)


class EmbeddedVideoField(models.URLField):
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 255
        super(EmbeddedVideoField, self).__init__(self, *args, **kwargs)

    def get_prep_value(self, value):
        return value.raw_url

    def to_python(self, value):
        if isinstance(value, EmbeddedVideo):
            return value

        return EmbeddedVideo(value)

    def formfield(self, **kwargs):
        defaults = {
            "label": "Embedded Video URL",
        }
        defaults.update(**kwargs)
        return super(EmbeddedVideoField, self).formfield(**defaults)

    def south_field_triple(self):
        from south.modelsinspector import introspector
        field_class = "%s.%s" % (self.__class__.__module__,
                self.__class__.__name__)
        args, kwargs = introspector(self)
        return (field_class, args, kwargs)



from datetime import datetime
from django.core.files import File
from django.conf import settings

from armstrong.dev.tests.utils import ArmstrongTestCase
from armstrong.dev.tests.utils.concrete import *
from armstrong.dev.tests.utils.users import *

import fudge


class ArmContentTestCase(ArmstrongTestCase):
    pass


def add_n_users_to_authors(authors, *users):
    for user in users:
        authors.users.add(user)


def add_profile_to(profile_class, *users):
    for user in users:
        profile = profile_class.objects.create(user=user)
        user._profile_cache = profile


def add_authors_to(model, *authors):
    for author in authors:
        model.authors.add(author)


def random_authored_model(klass, *authors):
    article = klass.objects.create()
    add_authors_to(article, *authors)
    return article



from .._utils import *

from ...admin import fieldsets


class FieldSetFactoryTestCase(ArmContentTestCase):
    fieldset_name = None

    @property
    def is_not_runnable(self):
        return self.fieldset_name is None

    def setUp(self):
        if self.is_not_runnable:
            return
        self.fieldset = getattr(fieldsets, self.fieldset_name)

    def test_expected_title(self):
        if self.is_not_runnable:
            return
        title, options = self.fieldset
        self.assertEqual(self.expected_title, title)

    def test_contains_expected_fields(self):
        if self.is_not_runnable:
            return
        fields = self.fieldset[1]["fields"]
        for field_name in self.expected_fields:
            self.assertTrue(field_name in fields, msg="checking for %s field")


class PublicationFieldSet(FieldSetFactoryTestCase):
    fieldset_name = 'PUBLICATION'
    expected_title = "Publication Information"
    expected_fields = ("pub_date", "pub_status", "sites", )


class AuthorsFieldSetTestCase(FieldSetFactoryTestCase):
    fieldset_name = 'AUTHORS'
    expected_title = "Author Information"
    expected_fields = ('authors', 'authors_override', 'authors_extra', )



from .._utils import *
import datetime
from django.db import models
from taggit.managers import TaggableManager

from armstrong.core.arm_sections.models import Section
from ..arm_content_support.models import ConcreteArticle
from ..arm_content_support.models import ConcreteCommentary
from ..arm_content_support.models import ConcreteContent
from ...models import ContentBase
from ...models import Section

now = datetime.datetime.now


class ContentBaseTestCase(ArmContentTestCase):
    @create_concrete_table
    def setUp(self):
        self.model = concrete(ContentBase)

    @destroy_concrete_table
    def tearDown(self):
        pass

    def test_has_a_title(self):
        self.assertModelHasField(self.model(), "title", models.CharField)

    def test_has_summary(self):
        self.assertModelHasField(self.model(), "summary", models.TextField)

    def test_has_slug(self):
        self.assertModelHasField(self.model(), "slug", models.SlugField)

    def test_has_tags(self):
        # Must create a full model here so we have a pk for tags to be
        # associated with
        model = self.model.objects.create(pub_date=now(),
                pub_status="Published")
        self.assertModelHasField(model, "tags", TaggableManager)

    def test_has_sections(self):
        model = self.model.objects.create(pub_date=now(),
                pub_status="Published")
        self.assertRelatedTo(model, "sections", Section, many=True)


class ConcreteContentBaseTestCase(ArmContentTestCase):
    def setUp(self):
        self.some_model = ConcreteArticle
        self.another_model = ConcreteCommentary

    def test_can_retrieve_all_models_as_part_of_parent_class(self):
        one = self.some_model.objects.create(pub_date=now(),
                pub_status="Published")
        two = self.another_model.objects.create(pub_date=now(),
                pub_status="Published")

        self.assertEqual(ConcreteContent.objects.all().count(), 2)
        pub_dates = [a.pub_date for a in ConcreteContent.objects.all()]
        self.assertTrue(one.pub_date in pub_dates)
        self.assertTrue(two.pub_date in pub_dates)

    def test_only_uses_one_query_to_get_subclasses(self):
        one = self.some_model.objects.create(pub_date=now(),
                pub_status="Published")
        two = self.another_model.objects.create(pub_date=now(),
                pub_status="Published")

        self.assertNotEqual(one.__class__, two.__class__, msg="sanity check")
        with self.assertNumQueries(1):
            subclass_queryset = ConcreteContent.objects.select_subclasses()
            [alt_one, alt_two] = subclass_queryset.all()
            self.assertEqual(alt_one.__class__, one.__class__)
            self.assertEqual(alt_two.__class__, two.__class__)

    def test_can_grab_by_full_slug(self):
        title = "Random Title %d" % random.randint(100, 200)
        slug = "random-%d" % random.randint(100, 200)
        article_slug = "some-random-article-slug-%d" % random.randint(100, 200)
        section = Section.objects.create(title="Random", slug=slug)
        m = self.some_model.objects.create(title=title, pub_date=now(),
                slug=article_slug, pub_status="Published")
        m.sections.add(section)

        article = self.some_model.with_section.get_by_slug("%s/%s" % (slug,
            article_slug))

        self.assertEqual(article.title, title)

    def test_can_grab_by_full_slug_if_in_nested_section(self):
        title = "Random Title %d" % random.randint(100, 200)
        slug = "random-%d" % random.randint(100, 200)
        article_slug = "some-random-article-slug-%d" % random.randint(100, 200)
        section = Section.objects.create(title="Random", slug=slug)
        child = Section.objects.create(title="Random Child", slug=slug,
                parent=section)
        m = self.some_model.objects.create(title=title, pub_date=now(),
                slug=article_slug, pub_status="Published")
        m.sections.add(child)

        article = self.some_model.with_section.get_by_slug("%s%s" %
                (child.full_slug, article_slug))

        self.assertEqual(article.title, title)



import urllib2

from .helpers import inject_defaults


class YouTubeBackend(object):
    type = "YouTube"

    def prepare(self, embed):
        embed.url = urllib2.urlparse.urlparse(embed.raw_url)
        embed.id = urllib2.urlparse.parse_qs(embed.url.query)['v'][0]

    @inject_defaults
    def embed(self, embed, width=None, height=None):
        # TODO: this should ultimately be a template so it can be swapped out
        return ''.join([
            '<iframe title="YouTube video player" ',
            'width="%s" height="%s" ' % (width, height),
            'src="http://www.youtube.com/embed/%s" ' % embed.id,
            'frameborder="0" allowfullscreen></iframe>'])



# coding=utf-8
from django.db import models
from .._utils import *

from ..arm_content_support.models import SimpleMixedinAuthorModel
from ..arm_content_support.models import SimpleProfile

from ..arm_content_support.forms import SimpleMixedinAuthorForm

from ...fields import AuthorsField
from ...fields import authors


class AuthorsMixinTestCase(ArmContentTestCase):
    def test_models_mixed_in_with_AuthorsMixin_have_an_authors_field(self):
        model = SimpleMixedinAuthorModel.objects.create()
        self.assertModelHasField(model, "authors", AuthorsField)

    def test_has_authors_extra_field(self):
        model = SimpleMixedinAuthorModel.objects.create()
        self.assertModelHasField(model, "authors_extra", models.CharField)

    def test_has_authors_override_field(self):
        model = SimpleMixedinAuthorModel.objects.create()
        self.assertModelHasField(model, "authors_override", models.CharField)

    def test_authors_fields_are_contain_all_users_when_cast_to_string(self):
        bob, alice = generate_random_staff_users()
        article = random_authored_model(SimpleMixedinAuthorModel, bob, alice)

        expected = "%s and %s" % (bob.get_full_name(), alice.get_full_name())
        self.assertEqual(str(article.authors), expected)

    def test_should_be_able_to_convert_to_unicode(self):
        bob, alice = generate_random_staff_users()
        bob.first_name = u"Bøb"
        bob.save()
        article = random_authored_model(SimpleMixedinAuthorModel, bob, alice)

        self.assertTrue(type(article.authors.__unicode__()) is unicode)
        expected = "%s and %s" % (bob.get_full_name(), alice.get_full_name())
        self.assertEqual(article.authors.__unicode__(), expected)

    def test_override_changes_the_string_output(self):
        override = "This is a random override %d" % random.randint(1000, 2000)
        bob, alice = generate_random_staff_users()

        article = random_authored_model(SimpleMixedinAuthorModel,
                bob, alice)
        article.authors_override = override

        self.assertEqual(str(article.authors), override)

    def test_extra_is_appended_to_the_end(self):
        extra = "extra %d data" % random.randint(1000, 2000)
        bob, alice = generate_random_staff_users()

        article = random_authored_model(SimpleMixedinAuthorModel,
                bob, alice)
        article.authors_extra = extra
        expected = "%s, %s %s" % (bob.get_full_name(),
                alice.get_full_name(), extra)
        self.assertEqual(str(article.authors), expected)

    def test_separates_all_but_last_two_users_with_commas(self):
        bob, alice = generate_random_staff_users()
        r = random.randint(1, 3)
        extra_authors = generate_random_staff_users(n=r)

        article = random_authored_model(SimpleMixedinAuthorModel, bob, alice,
                *extra_authors)

        self.assertEqual(r, str(article.authors).count(','))
        self.assertEqual(1, str(article.authors).count(' and '),
                msg="sanity check")

    def test_html_returns_plain_list_if_not_configured_with_profiles(self):
        bob, alice = generate_random_staff_users()
        expected = "%s and %s" % (bob.get_full_name(), alice.get_full_name())
        article = random_authored_model(SimpleMixedinAuthorModel, bob, alice)

        settings = fudge.Fake()
        settings.has_attr(AUTH_PROFILE_MODULE=None)
        with fudge.patched_context(authors, 'settings', settings):
            self.assertEqual(article.authors.html(), expected)

    def test_html_returns_string_with_html_links(self):
        bob, alice = generate_random_staff_users()
        article = random_authored_model(SimpleMixedinAuthorModel, bob, alice)
        add_profile_to(SimpleProfile, bob, alice)
        expected_html_links = [
                '<a href="/%s/">%s</a>' % (
                    bob.get_full_name().lower().replace(' ', '-'),
                    bob.get_full_name()),
                '<a href="/%s/">%s</a>' % (
                    alice.get_full_name().lower().replace(' ', '-'),
                    alice.get_full_name()),
        ]
        expected = ' and '.join(expected_html_links)
        self.assertEqual(article.authors.html(), expected)

    def test_alpha_spacing_of_extra(self):
        bob, alice = generate_random_staff_users()
        article = random_authored_model(SimpleMixedinAuthorModel,
                bob, alice)
        extra = "space expected before the s"
        article.authors_extra = extra
        expected = "%s, %s %s" % (bob.get_full_name(), alice.get_full_name(),
                extra)
        self.assertEqual(str(article.authors), expected)

    def test_no_spacing_with_non_alpha_on_extras(self):
        bob, alice = generate_random_staff_users()
        article = random_authored_model(SimpleMixedinAuthorModel,
                bob, alice)
        extra = ", no space expected before the s"
        article.authors_extra = extra
        expected = "%s, %s%s" % (bob.get_full_name(), alice.get_full_name(),
                extra)
        self.assertEqual(str(article.authors), expected)

    def test_mixed_fields_optional(self):
        fake_request_POST = {}
        form = SimpleMixedinAuthorForm(fake_request_POST)
        self.assertTrue(form.is_valid())



import urllib2

from .helpers import inject_defaults


class VimeoBackend(object):
    def prepare(self, embed):
        if not getattr(embed, "raw_url", False):
            return

        url = urllib2.urlparse.urlparse(embed.raw_url)
        if url.netloc[-9:] != "vimeo.com":
            return

        embed.url = url
        embed.id = url.path.lstrip("/")
        return True

    @inject_defaults
    def embed(self, embed, width=None, height=None):
        return "".join([
            '<iframe src="http://player.vimeo.com/video/%s' % embed.id,
            '?title=0&amp;byline=0&amp;portrait=0" ',
            'width="%s" height="%s" frameborder="0"></iframe>' % \
                    (width, height)
        ])



from .._utils import *

from ...fields.video import EmbeddedVideo
from ...fields.video import EmbeddedVideoField
from ..arm_content_support.models import SimpleMixedinVideoModel


class EmbeddedVideoMixinTestCase(ArmContentTestCase):
    def test_model_has_video_field(self):
        model = SimpleMixedinVideoModel.objects.create()
        self.assertModelHasField(model, "video", EmbeddedVideoField)
        self.assertTrue(isinstance(model.video, EmbeddedVideo))



from .base import BaseThumbnailMixin
from ...images.sorl import get_preset_thumbnail

class SorlThumbnailMixin(BaseThumbnailMixin):
    visual_field_name = 'image'

    def render_visual(self, preset_label, presets=None, defaults=None, *args, **kwargs):
        # TODO: Use a template for this.
        return '<img src="%s" />' % self.get_visual_thumbnail_url(preset_label,
            presets, defaults, *args, **kwargs)

    def get_visual_thumbnail_url(self, preset_label, presets=None, defaults=None, *args, **kwargs):
        image_file = getattr(self, self.visual_field_name)
        thumbnail_file = get_preset_thumbnail(image_file, preset_label,
            presets, defaults)
        return thumbnail_file.url



from copy import deepcopy

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


def get_preset_args(preset_label, presets=None, defaults=None):
    if presets is None:
        presets = getattr(settings, 'ARMSTRONG_PRESETS', {})

    if not preset_label in presets:
        raise ImproperlyConfigured('The "%s" preset hasn\'t been '
            'configured in your settings module.' % preset_label)
    
    if defaults is None:
        defaults = getattr(settings, 'ARMSTRONG_PRESET_DEFAULTS', {})
    args = deepcopy(defaults)
    args.update(presets[preset_label])
    return args



from django.contrib.auth.models import User
from django.db import models

import sorl.thumbnail

from ...fields import AuthorsField
from ...fields import EmbeddedVideoField
from ... import mixins
from ...mixins.images.sorl import SorlThumbnailMixin
from ...mixins.publication import PublishedManager
from ...models import ContentBase


from armstrong.apps.content.models import Content as ConcreteContent


class ConcreteArticle(ConcreteContent):
    pass


class ConcreteCommentary(ConcreteContent):
    pass


class SimpleVideoModel(models.Model):
    source = EmbeddedVideoField()


class SimpleMixedinVideoModel(mixins.EmbeddedVideoMixin, models.Model):
    pass


class SimpleAuthoredModel(models.Model):
    authors = AuthorsField()


class SimpleMixedinAuthorModel(mixins.AuthorsMixin, models.Model):
    pass


class AuthoredModelWithContentionalOverride(models.Model):
    authors = AuthorsField()
    authors_override = models.CharField(max_length=100)


class AuthoredModelWithConfiguredOverride(models.Model):
    authors = AuthorsField(override_field_name='custom_override')
    custom_override = models.CharField(max_length=100)


class AuthoredModelWithContentionalExtra(models.Model):
    authors = AuthorsField()
    authors_extra = models.CharField(max_length=100)


class AuthoredModelWithConfiguredExtra(models.Model):
    authors = AuthorsField(extra_field_name='custom_extra')
    custom_extra = models.CharField(max_length=100)


class SimpleProfile(models.Model):
    user = models.OneToOneField(User)

    def get_absolute_url(self):
        return '/%s/' % self.user.get_full_name().lower().replace(' ', '-')


class SorlImage(SorlThumbnailMixin, models.Model):
    image = sorl.thumbnail.ImageField(upload_to='images/')


class ContentOne(ContentBase):
    """
    This makes sure we have at least one model with sections and its reverse
    name.
    """
    pass

class ContentTwo(ContentBase):
    """
    This makes sure we have a second ContentBase sub-class that explicitly
    breaks the reverse name of the sections field.
    """
    pass



from django.contrib.auth.models import User
from django.test import TestCase

from .models import SimpleProfile


class SimpleProfileTestCase(TestCase):
    def test_get_absolute_url_slugifies_users_name(self):
        user = User.objects.create(username="travis", first_name="Travis",
                last_name="Swicegood")
        profile = SimpleProfile(user=user)
        self.assertEqual("/travis-swicegood/", profile.get_absolute_url())



# coding=utf-8
from django.db import models
from .._utils import *

from ..arm_content_support.models import ConcreteArticle
from ...models import ContentBase

import datetime

class PublicationManagerTestCase(ArmContentTestCase):
    def setUp(self):
        self.published = ConcreteArticle.objects.create(
                title="Published",
                pub_date=datetime.datetime.now()-datetime.timedelta(days=1),
                pub_status='P'
            )
        self.draft_art = ConcreteArticle.objects.create(
                title="Not Published",
                pub_date=datetime.datetime.now()-datetime.timedelta(days=1),
                pub_status='D'
            )
        self.scheduled = ConcreteArticle.objects.create(
                title="Future Published",
                pub_date=datetime.datetime.now()+datetime.timedelta(days=1),
                pub_status='P'
            )

    def test_ContentBase_subclasses_have_published_manager(self):
        class ConcreteContent(ContentBase):
            pass
        self.assertTrue(hasattr(ConcreteContent, 'published'))

    def test_subclasses_of_concrete_classes_have_published_manager(self):
        class ConcreteContent(ContentBase):
            pass
        class ConcreteArticle(ConcreteContent):
            pass
        self.assertTrue(hasattr(ConcreteArticle, 'published'))

    def test_published_manager_only_pulls_published_content(self):
        all_published = ConcreteArticle.published.all().select_subclasses()
        self.assertIn(self.published, all_published)
        self.assertNotIn(self.draft_art, all_published)
        self.assertNotIn(self.scheduled, all_published)

    def test_is_published(self):
        self.assertTrue(self.published.is_published)
        self.assertFalse(self.draft_art.is_published)
        self.assertFalse(self.scheduled.is_published)



from django.db import models
from django.contrib.sites.models import Site
from datetime import datetime
from model_utils.managers import InheritanceManager

PUB_STATUS_CHOICES = (
    ('D', 'Draft'),
    ('E', 'Edit'),
    ('P', 'Published'),
    ('T', 'Trash'),
)

PUB_STATUSES = dict((pair[1], pair[0]) for pair in PUB_STATUS_CHOICES)


class SimplePublicationMixin(models.Model):
    pub_date = models.DateTimeField(db_index=True)
    pub_status = models.CharField((u'Publication status'), max_length=1,
        choices=PUB_STATUS_CHOICES, help_text=(
            u'Only published items will appear on the site'))

    @property
    def is_published(self):
        return self.pub_date <= datetime.now() \
            and self.pub_status == "P"

    class Meta:
        abstract = True


class PublicationMixin(SimplePublicationMixin, models.Model):
    sites = models.ManyToManyField(Site)

    class Meta:
        abstract = True


class PublishedManager(InheritanceManager):
    """ Returns published objects where the pub_date has already passed """
    def get_query_set(self):
        return super(PublishedManager, self).get_query_set()\
                .filter(pub_date__lte=datetime.now())\
                .filter(pub_status="P")



from django.conf import settings
from django.db import models
from django.db.models.query import QuerySet
from model_utils.managers import InheritanceManager
from taggit.managers import TaggableManager

from armstrong.core.arm_sections.managers import SectionSlugManager
from armstrong.core.arm_sections.models import Section
from armstrong.core.arm_access.mixins import AccessMixin

from .mixins import AuthorsMixin, PublicationMixin
from .mixins.publication import PublishedManager


class ContentBase(AuthorsMixin, PublicationMixin, AccessMixin, models.Model):
    """
    The base class providing the basic "armstrong" behavior for a model.

    This is provided as a way to handle cross-model querying.  For example, you
    can use this to query across Article and Video models assuming they both
    extend from a concrete implementation of this class.

    This is *not* a concrete implementation.  This is to avoid having any
    tables created that are not needed.  `armstrong.apps.content`_ provides a
    concrete implementation of this if you want to use it without defining your
    own base Content model.

    .. _armstrong.apps.content: http://github.com/armstrongcms/armstrong.apps.content
    """

    title = models.CharField(max_length=255)
    summary = models.TextField()
    # TODO: Increase max length for SlugField (see Issue #43)
    slug = models.SlugField()

    sections = models.ManyToManyField(Section, null=True, blank=True,
            related_name="%(app_label)s_%(class)s_alternates")

    tags = TaggableManager(blank=True)
    with_section = SectionSlugManager(section_field="sections")

    objects = InheritanceManager()
    published = PublishedManager()

    # TODO: add required primary section
    # TODO: add secondary sections
    class Meta:
        abstract = True

    def get_absolute_url(self):
        urlconf = __import__(settings.ROOT_URLCONF,
                             globals(),
                             locals(),
                             ['get_url_for_model'])
        return urlconf.get_url_for_model(self)

    def __unicode__(self):
        return self.title



# coding=utf-8
from django.contrib.auth.models import User
from django.db import models
from django.utils import unittest
from django.template import Template, Context
import fudge
try:
    import south
except ImportError:
    south = False

from ..arm_content_support.models import AuthoredModelWithConfiguredOverride
from ..arm_content_support.models import AuthoredModelWithContentionalOverride
from ..arm_content_support.models import AuthoredModelWithConfiguredExtra
from ..arm_content_support.models import AuthoredModelWithContentionalExtra
from ..arm_content_support.models import SimpleAuthoredModel
from ..arm_content_support.models import SimpleProfile
from .._utils import *

from ...fields import authors


class AuthorsFieldTestCase(ArmContentTestCase):
    def test_authors_can_have_one_author(self):
        [bob, ] = generate_random_staff_users(n=1)
        article = SimpleAuthoredModel.objects.create()
        article.authors.add(bob)

        self.assertEqual(bob.get_full_name(), str(article.authors))

    def test_authors_fields_are_contain_all_users_when_cast_to_string(self):
        bob, alice = generate_random_staff_users()
        article = random_authored_model(SimpleAuthoredModel, bob, alice)

        expected = "%s and %s" % (bob.get_full_name(), alice.get_full_name())
        self.assertEqual(str(article.authors), expected)

    def test_should_be_able_to_convert_to_unicode(self):
        bob, alice = generate_random_staff_users()
        bob.first_name = u"Bøb"
        bob.save()
        article = random_authored_model(SimpleAuthoredModel, bob, alice)

        self.assertTrue(type(article.authors.__unicode__()) is unicode)
        expected = "%s and %s" % (bob.get_full_name(), alice.get_full_name())
        self.assertEqual(article.authors.__unicode__(), expected)

    def test_override_changes_the_string_output(self):
        override = "This is a random override %d" % random.randint(1000, 2000)
        bob, alice = generate_random_staff_users()

        article = random_authored_model(AuthoredModelWithContentionalOverride,
                bob, alice)
        article.authors_override = override

        self.assertEqual(str(article.authors), override)

    def test_override_is_ignored_if_empty(self):
        empty_override = ""
        bob, alice = generate_random_staff_users()

        article = random_authored_model(AuthoredModelWithContentionalOverride,
                bob, alice)
        article.authors_override = empty_override

        expected = "%s and %s" % (bob.get_full_name(), alice.get_full_name())
        self.assertEqual(str(article.authors), expected)

    def test_override_can_be_configured(self):
        # TODO: dynamically generate this model and use a random name
        override = "This is a random override %d" % random.randint(1000, 2000)
        bob, alice = generate_random_staff_users()

        article = random_authored_model(AuthoredModelWithConfiguredOverride,
                bob, alice)
        add_authors_to(article, bob, alice)
        article.custom_override = override

        self.assertEqual(str(article.authors), override)

    def test_extra_is_appended_to_the_end(self):
        extra = "extra %d data" % random.randint(1000, 2000)
        bob, alice = generate_random_staff_users()

        article = random_authored_model(AuthoredModelWithContentionalExtra,
                bob, alice)
        article.authors_extra = extra
        expected = "%s, %s %s" % (bob.get_full_name(),
                alice.get_full_name(), extra)
        self.assertEqual(str(article.authors), expected)

    def test_extra_field_can_be_configured(self):
        extra = "extra %d data" % random.randint(1000, 2000)
        bob, alice = generate_random_staff_users()

        article = random_authored_model(AuthoredModelWithConfiguredExtra,
                bob, alice)
        article.custom_extra = extra
        expected = "%s, %s %s" % (bob.get_full_name(),
                alice.get_full_name(), extra)
        self.assertEqual(str(article.authors), expected)

    def test_separates_all_but_last_two_users_with_commas(self):
        bob, alice = generate_random_staff_users()
        r = random.randint(1, 3)
        extra_authors = generate_random_staff_users(n=r)

        article = random_authored_model(SimpleAuthoredModel, bob, alice,
                *extra_authors)

        self.assertEqual(r, str(article.authors).count(','))
        self.assertEqual(1, str(article.authors).count(' and '),
                msg="sanity check")

    def test_html_returns_plain_list_if_not_configured_with_profiles(self):
        bob, alice = generate_random_staff_users()
        expected = "%s and %s" % (bob.get_full_name(), alice.get_full_name())
        article = random_authored_model(SimpleAuthoredModel, bob, alice)

        settings = fudge.Fake()
        settings.has_attr(AUTH_PROFILE_MODULE=None)
        with fudge.patched_context(authors, 'settings', settings):
            self.assertEqual(article.authors.html(), expected)

    def test_html_returns_string_with_html_links(self):
        bob, alice = generate_random_staff_users()
        article = random_authored_model(SimpleAuthoredModel, bob, alice)
        add_profile_to(SimpleProfile, bob, alice)
        expected_html_links = [
                '<a href="/%s/">%s</a>' % (
                    bob.get_full_name().lower().replace(' ', '-'),
                    bob.get_full_name()),
                '<a href="/%s/">%s</a>' % (
                    alice.get_full_name().lower().replace(' ', '-'),
                    alice.get_full_name()),
        ]
        expected = ' and '.join(expected_html_links)
        self.assertEqual(article.authors.html(), expected)

    def test_alpha_spacing_of_extra(self):
        bob, alice = generate_random_staff_users()
        article = random_authored_model(AuthoredModelWithContentionalExtra,
                bob, alice)
        extra = "space expected before the s"
        article.authors_extra = extra
        expected = "%s, %s %s" % (bob.get_full_name(), alice.get_full_name(),
                extra)
        self.assertEqual(str(article.authors), expected)

    def test_no_spacing_with_non_alpha_on_extras(self):
        bob, alice = generate_random_staff_users()
        article = random_authored_model(AuthoredModelWithContentionalExtra,
                bob, alice)
        extra = ", no space expected before the s"
        article.authors_extra = extra
        expected = "%s, %s%s" % (bob.get_full_name(), alice.get_full_name(),
                extra)
        self.assertEqual(str(article.authors), expected)

    @unittest.skipIf(south is False, "south not installed")
    def test_provides_south_field_triple(self):
        field = authors.AuthorsField()
        expected = (
            "%s.%s" % (field.__class__.__module__, field.__class__.__name__),
            [],
            {
                "to": "orm['auth.User']",
                "override_field_name": "'authors_override'",
                "extra_field_name": "'authors_extra'",
                "symmetrical": "False",
            })
        self.assertEqual(field.south_field_triple(), expected)

    def test_defaults_to_being_related_to_base_user(self):
        field = authors.AuthorsField()
        self.assertEqual(field.rel.to, User)

    def test_can_relate_to_custom_user(self):
        class MyUser(models.Model):
            pass

        field = authors.AuthorsField(to=MyUser)
        self.assertEqual(field.rel.to, MyUser)

    def test_can_render_in_templates(self):
        [bob, ] = generate_random_staff_users(n=1)
        article = SimpleAuthoredModel.objects.create()
        article.authors.add(bob)
        t = Template("{{ article.authors }}")
        context = Context({"article": article})
        self.assertEqual(bob.get_full_name(), t.render(context))



class AuthorsDescriptorTestCase(ArmContentTestCase):
    def test_does_not_choke_on_empty_instance(self):
        try:
            authors_field = SimpleAuthoredModel.authors
            self.assertTrue(True, "Was able to look at authors on the model")
        except AttributeError, e:
            self.fail("Should not have raised an exception: %s" % e)

    def test_returns_descriptor_when_retrieved_off_of_model(self):
        authors_field = SimpleAuthoredModel.authors
        self.assertTrue(isinstance(authors_field, authors.AuthorsDescriptor))

    def test_can_accept_being_set_to_a_list(self):
        article = SimpleAuthoredModel.objects.create()
        staff = generate_random_staff_users(n=2)
        article.authors = staff
        self.assertTrue(type(article.authors) is not list)

    def test_has_the_same_users_after_being_set_to_list(self):
        article = SimpleAuthoredModel.objects.create()
        staff = generate_random_staff_users(n=2)
        article.authors = staff

        self.assertEqual(article.authors.all().count(), len(staff))
        for author in article.authors.all():
            self.assertTrue(author in staff)
